package com.gic.haoban.manage.service.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseMapper;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseRelatedMapper;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterpriseRelated;
import com.gic.haoban.manage.service.pojo.bo.WxEnterpriseRelationBO;
import com.gic.haoban.manage.service.service.WxEnterpriseRelatedService;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

@Service
public class WxEnterpriseRelatedServiceImpl implements WxEnterpriseRelatedService {

    private static final Logger logger = getLogger(WxEnterpriseRelatedServiceImpl.class);

	@Autowired
	private WxEnterpriseRelatedMapper mapper;

    @Autowired
    private WxEnterpriseMapper wxEnterpriseMapper;

	@Override
	public TabHaobanWxEnterpriseRelated getByGicEnterpriseIdByEnterpriseRelatedId(String wxEnterpriseRelatedId) {
		
		return mapper.selectByPrimaryKey(wxEnterpriseRelatedId);
	}


	@Override
	public List<TabHaobanWxEnterpriseRelated> getByWxEnterpriseId(String wxEnterpriseId) {
		List<TabHaobanWxEnterpriseRelated> list = mapper.listByWxenterpriseId(wxEnterpriseId);
		return list;
	}

    @Override
    public List<TabHaobanWxEnterprise> listByEnterpriseId(String enterpriseId) {
        List<String> eids = new ArrayList<>();
        eids.add(enterpriseId);
        List<TabHaobanWxEnterpriseRelated> relateds = mapper.listEnterpriseByEnterpriseIds(eids);
        if (CollectionUtils.isEmpty(relateds)) {
            return null;
        }
        Set<String> wxEnterpriseIds = relateds.stream().map(tab -> tab.getWxEnterpriseId()).collect(Collectors.toSet());
        return wxEnterpriseMapper.listByIds(wxEnterpriseIds);
    }

    @Override
    public TabHaobanWxEnterpriseRelated getByEnterpriseId(String wxEnterpriseId, String enterpriseId) {
        return mapper.findOneByEIdAndWxEid(enterpriseId, wxEnterpriseId);
    }

    @Override
    public void setClerkEdit(String id, int clerkEditFlag) {
        TabHaobanWxEnterpriseRelated record = new TabHaobanWxEnterpriseRelated();
        record.setClerkEditFlag(clerkEditFlag);
        record.setWxEnterpriseRelatedId(id);
        record.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(record);
    }
    
    @Override
    public int getRelationCount(String enterpriseId) {
        List<TabHaobanWxEnterpriseRelated> list = this.mapper.listByEnterpriseId(enterpriseId) ;
        return list.size();
    }
    
    @Override
    public WxEnterpriseRelationBO getHeadEnterprise(String enterpriseId) {
        TabHaobanWxEnterpriseRelated root = this.mapper.findRootWxEnterpriseByEnterpriseId(enterpriseId) ;
        if(null != root) {
            return EntityUtil.changeEntityByJSON(WxEnterpriseRelationBO.class, root);
        }
        return null;
    }
}
