package com.gic.haoban.manage.service.service.out.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.api.constant.EnterpriseServiceEnum;
import com.gic.enterprise.api.service.EnterpriseUseForbidService;
import com.gic.haoban.manage.api.dto.QwPendingIdDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.service.QwFriendApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.pojo.bo.WxEnterpriseRelationBO;
import com.gic.haoban.manage.service.service.WxEnterpriseRelatedService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.util.QwInterfaceLimitEnum;
import com.gic.haoban.manage.service.util.QwTimesUtils;
import com.gic.wechat.api.dto.qywx.PendingIdDTO;
import com.gic.wechat.api.service.qywx.QywxExternalUserService;

@Service("qwFriendApiService")
public class QwFriendApiServiceImpl implements QwFriendApiService {

    private static final Logger log = LogManager.getLogger(QwFriendApiService.class);

    @Autowired
    private EnterpriseUseForbidService enterpriseUseForbidService;
    @Autowired
    private QywxExternalUserService qywxExternalUserService;
    @Autowired
    private WxEnterpriseRelatedService wxEnterpriseRelatedService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private Config config;

    @Override
    public ServiceResponse<QwPendingIdDTO> getPendingIdByUnionid(String enterpriseId, String unionid, String openid) {
        String suiteid = config.getWxSuiteid();
        ServiceResponse<WxEnterpriseQwDTO> corpidResp = this.checkAndGetCorpid(enterpriseId);
        if (!corpidResp.isSuccess()) {
            return ServiceResponse.failure(corpidResp.getCode(), corpidResp.getMessage());
        }
        WxEnterpriseQwDTO qwDTO = corpidResp.getResult();
        String wxEnterpriseId = qwDTO.getWxEnterpriseId();
        
        boolean isLimit = QwTimesUtils.isLimit(wxEnterpriseId, enterpriseId,QwInterfaceLimitEnum.GET_PENDING_ID);
        if (isLimit) {
            return ServiceResponse.failure(HaoBanErrCode.ERR_2000.getCode(), HaoBanErrCode.ERR_2000.getMsg());
        }
        String corpid = qwDTO.getThirdCorpid();
        ServiceResponse<PendingIdDTO> qwResp = this.qywxExternalUserService.unionidToExternalUserid(corpid, suiteid,
                unionid, openid, 1);
        QwTimesUtils.addTimes(wxEnterpriseId, QwInterfaceLimitEnum.GET_PENDING_ID);
        log.info("获取pendingid,参数={},返回={}", unionid, JSON.toJSONString(qwResp));
        if (!qwResp.isSuccess()) {
            return ServiceResponse.failure(qwResp.getCode(), qwResp.getMessage());
        }
        QwPendingIdDTO dto = new QwPendingIdDTO();
        dto.setExternalUserId(qwResp.getResult().getExternal_userid());
        dto.setPendingId(qwResp.getResult().getPending_id());
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<QwPendingIdDTO> getPendingIdByExternalid(String enterpriseId, String externalUserId) {
        String suiteid = config.getWxSuiteid();
        ServiceResponse<WxEnterpriseQwDTO> corpidResp = this.checkAndGetCorpid(enterpriseId);
        if (!corpidResp.isSuccess()) {
            return ServiceResponse.failure(corpidResp.getCode(), corpidResp.getMessage());
        }
        WxEnterpriseQwDTO qwDTO = corpidResp.getResult();
        String corpid = qwDTO.getThirdCorpid();
        ServiceResponse<List<PendingIdDTO>> qwResp = this.qywxExternalUserService.externalUseridToPendingId(corpid,
                suiteid, Arrays.asList(externalUserId));
        log.info("获取pendingid,参数={},返回={}", externalUserId, JSON.toJSONString(qwResp));
        if (!qwResp.isSuccess()) {
            return ServiceResponse.failure(qwResp.getCode(), qwResp.getMessage());
        }
        List<PendingIdDTO> list = qwResp.getResult();
        if (CollectionUtils.isNotEmpty(list)) {
            QwPendingIdDTO dto = new QwPendingIdDTO();
            dto.setExternalUserId(list.get(0).getExternal_userid());
            dto.setPendingId(list.get(0).getPending_id());
            return ServiceResponse.success(dto);
        }
        return ServiceResponse.failure("9999", "企微未查询到数据");
    }

    @Override
    public ServiceResponse<List<QwPendingIdDTO>> listPendingIdByExternalid(String enterpriseId,
            List<String> externalUserIdList) {
        if (CollectionUtils.isEmpty(externalUserIdList) || externalUserIdList.size() > 1000) {
            return ServiceResponse.failure("9999", "外部联系人不能为空且每次最多1000条");
        }
        String suiteid = config.getWxSuiteid();
        ServiceResponse<WxEnterpriseQwDTO> corpidResp = this.checkAndGetCorpid(enterpriseId);
        if (!corpidResp.isSuccess()) {
            return ServiceResponse.failure(corpidResp.getCode(), corpidResp.getMessage());
        }
        WxEnterpriseQwDTO qwDTO = corpidResp.getResult();
        String corpid = qwDTO.getThirdCorpid();
        ServiceResponse<List<PendingIdDTO>> qwResp = this.qywxExternalUserService.externalUseridToPendingId(corpid,
                suiteid, externalUserIdList);
        log.info("获取pendingid,参数={},返回={}", externalUserIdList, JSON.toJSONString(qwResp));
        if (!qwResp.isSuccess()) {
            return ServiceResponse.failure(qwResp.getCode(), qwResp.getMessage());
        }
        List<PendingIdDTO> list = qwResp.getResult();
        if (CollectionUtils.isNotEmpty(list)) {
            List<QwPendingIdDTO> retList = new ArrayList<>();
            for (PendingIdDTO item : list) {
                QwPendingIdDTO dto = new QwPendingIdDTO();
                dto.setExternalUserId(item.getExternal_userid());
                dto.setPendingId(item.getPending_id());
                retList.add(dto);
            }
            return ServiceResponse.success(retList);
        }
        return ServiceResponse.failure("9999", "企微未查询到数据");
    }

    // 判断企业是否能用企微
    private ServiceResponse<WxEnterpriseQwDTO> checkAndGetCorpid(String enterpriseId) {
        WxEnterpriseRelationBO enterpriseRelation = this.wxEnterpriseRelatedService.getHeadEnterprise(enterpriseId);
        if (null == enterpriseRelation) {
            // 非总部或不存在
            return ServiceResponse.failure("9999", "请确认商户是否有效且是总部");
        }
        Integer code = this.enterpriseUseForbidService.selectPermissionStatus(enterpriseId,
                EnterpriseServiceEnum.HAO_BAN.getRightMenuCode());
        log.info("好办是否到期停用={},{}", enterpriseId, code);
        if (code != 0 && code != 1) {
            return ServiceResponse.failure("9999", "商户已停用");
        }
        int memberOpenCardlag = enterpriseRelation.getMemberOpenCardFlag();
        if (memberOpenCardlag != 0) {
            // 不在同一开放平台
            return ServiceResponse.failure("9999", "不在同一开放平台");
        }
        String wxEnterpriseId = enterpriseRelation.getWxEnterpriseId();
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        return ServiceResponse.success(qwDTO);
    }

}
