package com.gic.haoban.manage.service.util;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RAtomicLong;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.service.CustomSettingApiService;
import com.gic.haoban.common.utils.DingUtils;
import com.gic.haoban.manage.service.pojo.bo.QwLimitTimesBO;
import com.gic.redis.data.util.RedisUtil;

@Component
public class QwTimesUtils {

    private static Logger logger = LogManager.getLogger(QwTimesUtils.class);
    private static CustomSettingApiService customSettingApiService;

    public static EnterpriseService enterpriseService;

    @Autowired
    public void setCustomSettingApiService(CustomSettingApiService customSettingApiService) {
        QwTimesUtils.customSettingApiService = customSettingApiService;
    }

    @Autowired
    public void setEnterpriseService(EnterpriseService enterpriseService) {
        QwTimesUtils.enterpriseService = enterpriseService;
    }

    public static boolean isLimit(String wxEnterpriseId, String enterpriseId, QwInterfaceLimitEnum obj) {
        QwLimitTimesBO setTimeBO = getSettingTimes(wxEnterpriseId, obj);
        logger.info("获取配置次数={},wxEnterpriseId={}", JSON.toJSONString(setTimeBO), wxEnterpriseId);
        // 先判断月
        String interfaceName = obj.getName();
        Date now = new Date();
        String dateStr = getDateStr(now);
        String keyMonth = getCacheKey(wxEnterpriseId, interfaceName, dateStr.substring(0, 6));
        boolean monthResult = timesCheck(1, keyMonth, obj, wxEnterpriseId, enterpriseId, now, setTimeBO);
        if (monthResult) {
            logger.info("月达到限制={}", keyMonth);
            return true;
        }
        // 天
        String keyDay = getCacheKey(wxEnterpriseId, interfaceName, dateStr.substring(0, 8));
        boolean dayResult = timesCheck(2, keyDay, obj, wxEnterpriseId, enterpriseId, now, setTimeBO);
        if (dayResult) {
            logger.info("天达到限制={}", keyDay);
            return true;
        }
        // 小时
        String keyHour = getCacheKey(wxEnterpriseId, interfaceName, dateStr.substring(0, 10));
        boolean hourResult = timesCheck(3, keyHour, obj, wxEnterpriseId, enterpriseId, now, setTimeBO);
        if (hourResult) {
            logger.info("小时达到限制={}", keyHour);
            return true;
        }
        return false;
    }

    private static boolean timesCheck(int type, String cacheKey, QwInterfaceLimitEnum obj, String wxEnterpriseId,
            String enterpriseId, Date now, QwLimitTimesBO setTimeBO) {
        Integer doneTimes = RedisUtil.getCache(cacheKey, Integer.class);
        logger.info("获取已调次数={},cacheKey={}", doneTimes, cacheKey);
        if (null == doneTimes) {
            return false;
        }
        int setTimes = 0;
        switch (type) {
        case 1:
            setTimes = setTimeBO.getMonth();
            break;
        case 2:
            setTimes = setTimeBO.getDay();
            break;
        case 3:
            setTimes = setTimeBO.getHour();
            break;
        default:
            break;
        }
        sendAlert(type, doneTimes, setTimes, obj, enterpriseId, now);
        if (doneTimes >= setTimes) {
            return true;
        }
        return false;
    }

    private static QwLimitTimesBO getSettingTimes(String wxEnterpriseId, QwInterfaceLimitEnum obj) {
        String key = String.format("%s_%s", obj.getName(), wxEnterpriseId);
        QwLimitTimesBO times = getTimes(key);
        if (null == times) {
            return getTimes(obj.getName());
        }
        return times;
    }

    private static QwLimitTimesBO getTimes(String key) {
        JSONObject searchParamMap = new JSONObject();
        searchParamMap.put("key", key);
        ServiceResponse<Map<String, Object>> resp = customSettingApiService.getDetail("tab_gic_map",
                searchParamMap.toJSONString());
        if (resp.isSuccess() && null != resp.getResult()) {
            QwLimitTimesBO times = new QwLimitTimesBO();
            times = JSON.parseObject(resp.getResult().get("value").toString(), QwLimitTimesBO.class);
            logger.info("获取默认配置次数={},{}", times, key);
            return times;
        }
        return null;
    }

    private static void sendAlert(int type, int doneTimes, int setTimes, QwInterfaceLimitEnum obj, String enterpriseId,
            Date now) {
        // 调用次数达上线
        String time = null;
        String title = "已达上限";
        boolean sendFlag = false;
        if (doneTimes >= setTimes) {
            sendFlag = true;
            if (type == 1) {
                time = DateUtil.dateToStr(now, "yyyy年MM月");
            }
            if (type == 2) {
                time = DateUtil.dateToStr(now, "yyyy年MM月dd日");
            }
            if (type == 3) {
                time = DateUtil.dateToStr(now, "yyyy年MM月dd日 HH:mm:ss");
            }
        } else if (type == 1 || type == 2) {
            switch (type) {
            case 1:
                setTimes = setTimes - 30000;
                break;
            case 2:
                setTimes = setTimes - 300000;
                break;
            default:
                break;
            }
            if (doneTimes >= setTimes) {
                sendFlag = true;
                if (type == 1) {
                    time = DateUtil.dateToStr(now, "yyyy年MM月");
                }
                if (type == 2) {
                    DateUtil.dateToStr(now, "yyyy年MM月dd日");
                }
                title = "即将达到可调用次数";
            }
        }
        if (sendFlag) {
            EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(enterpriseId);
            String brandName = enterpriseDTO.getBrandName();
            // 达到上限时文案
            String msg = String.format("告警【%s】%s调用在【%s】%s", brandName, obj.getRemark(), time, title);
            DingUtils.send(msg, false);
        }
    }

    public static void addTimes(String wxEnterpriseId, QwInterfaceLimitEnum obj) {
        String interfaceName = obj.getName();
        Date now = new Date();
        String dateStr = getDateStr(now);
        String keyHour = getCacheKey(wxEnterpriseId, interfaceName, dateStr.substring(0, 10));
        addTimes(3, keyHour);
        String keyDay = getCacheKey(wxEnterpriseId, interfaceName, dateStr.substring(0, 8));
        addTimes(2, keyDay);
        String keyMonth = getCacheKey(wxEnterpriseId, interfaceName, dateStr.substring(0, 6));
        addTimes(1, keyMonth);
    }

    private static String getCacheKey(String wxEnterpriseId, String interfacName, String dateStr) {
        return new StringBuilder(wxEnterpriseId).append(interfacName).append(dateStr).toString();
    }

    // 1月 2天 3小时
    private static void addTimes(int type, String key) {
        RAtomicLong cache = RedisUtil.getRedisClient().getAtomicLong(key);
        long val = cache.addAndGet(1);
        if (val <= 1) {
            switch (type) {
            case 1:
                cache.expire(31, TimeUnit.DAYS);
                break;
            case 2:
                cache.expire(1, TimeUnit.DAYS);
                break;
            case 3:
                cache.expire(1, TimeUnit.HOURS);
                break;
            default:
                break;
            }
        }
    }

    private static String getDateStr(Date now) {
        String dateStr = DateUtil.dateToStr(now, "yyyyMMddHHmmss");
        return dateStr;
    }
}
